package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.net.Uri
import com.google.firebase.database.DatabaseReference
import com.google.firebase.storage.FirebaseStorage
import cz.uhk.pavolpodstreleny.schoolproject.data.await
import java.util.*

/**
 * Created by pavol.podstreleny on 11.01.2018.
 *
 * Abstract class that extract basic Admin CRUD methods and Transformation data functionality
 *
 */

abstract class AdminRepository : BaseRepository() {

    private val mStorage = FirebaseStorage.getInstance()


    /**
     * General method for deletingImages from FirebaseStorage
     *
     * @param filePath path of the file in storage
     *
     * @return boolean value which indicate if image was deleted or not
     */

    protected suspend fun deleteImage(filePath: String): Boolean {
        val photoRef = mStorage.reference.child(filePath)
        try {
            photoRef.delete().await()
            return true
        } catch (e: Exception) {
            return false
        }
    }

    /**
     * General method for deleting Data from FirebaseDatabase
     *
     * @param firstChildPath path that points to certain data in database
     * @param secondChildPath path that points to certain data in database combined with firstChildPath
     *
     * @return boolean value indicated if operation was successful
     */

    protected suspend fun deleteData(firstChildPath: String, secondChildPath: String? = null): Boolean {
        val dataRef: DatabaseReference
        if (secondChildPath == null) {
            dataRef = mDatabase.reference.child(firstChildPath)
        } else {
            dataRef = mDatabase.reference.child(firstChildPath).child(secondChildPath)
        }

        try {
            dataRef.removeValue().await()
            return true
        } catch (e: Exception) {
            return false
        }
    }

    /**
     * General method for adding image to FirebaseStorage
     *
     * @param imageUri uri adress of uploaded image from phone
     *
     * @return uri adress where you can access image or null if operation fails
     *
     */

    protected suspend fun addImage(imageFolderPath: String, imageUri: Uri): Uri? {
        val photoRef = mStorage.reference.child(imageFolderPath).child(imageUri.lastPathSegment + Math.random())
        try {
            val savedPhoto = photoRef.putFile(imageUri).await()
            return savedPhoto.downloadUrl
        } catch (e: Exception) {
            return null
        }
    }


    /**
     * General method for adding data to FireabseDatabase
     *
     * @param newDataPath path where new child will be added
     * @param hashMap contains values which will be added to database
     *
     * @return boolean value indicated if operation was successful
     */
    protected suspend fun addData(newDataPath: String, hashMap: HashMap<String, Any>): Boolean {
        val dataRef = mDatabase.reference.child(newDataPath)
        try {
            dataRef.push().setValue(hashMap)
            return true
        } catch (e: Exception) {
            return false
        }
    }



    /**
     * General method for editing data in FirebaseDatabase
     *
     * @param firstChildPath path that points to certain data in database
     * @param secondChildPath path that points to certain data in database combined with firstChildPath
     * @param data data that suppose to be updated
     *
     */
    protected suspend fun editData(firstChildPath: String, secondChildPath: String, data: HashMap<String, Any>): Boolean {
        val reference = mDatabase.reference.child(firstChildPath).child(secondChildPath)

        try {
            reference.updateChildren(data)
            return true
        } catch (e: Exception) {
            return false
        }
    }

}