package cz.uhk.pavolpodstreleny.schoolproject.data.repositories

import android.arch.lifecycle.LiveData
import android.arch.lifecycle.MediatorLiveData
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.database.DataSnapshot
import cz.uhk.pavolpodstreleny.schoolproject.R
import cz.uhk.pavolpodstreleny.schoolproject.data.*
import cz.uhk.pavolpodstreleny.schoolproject.model.Badge
import cz.uhk.pavolpodstreleny.schoolproject.model.Task
import java.util.*
import kotlin.collections.HashMap

/**
 * Created by pavol.podstreleny on 21.01.2018.
 */
class TaskAdminRepository : AdminRepository() {


    suspend fun addTask(description: String, exerciseID: String?, options: HashMap<String, Boolean>): FirebaseResult {
        if (exerciseID == null || options.isEmpty()) {
            return FirebaseResult(false, R.string.task_admin_page_task_added_error)
        }

        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_TASKS_EXERCISE, exerciseID)
        map.put(DatabaseContract.TABLE_TASKS_DESCRIPTION, description)
        map.put(DatabaseContract.TABLE_TASKS_PHOTOURL, "null")
        map.put(DatabaseContract.TABLE_TASKS_CHOICES, options)

        if (addData(DatabaseContract.TABLE_TASKS_NAME, map)) {
            return FirebaseResult(true, R.string.task_admin_page_task_added)
        } else {
            return FirebaseResult(false, R.string.task_admin_page_task_added_error)
        }

    }

    suspend fun addBadge(userID: String, badgeID: String): FirebaseResult {
        val map = HashMap<String, Any>()
        map.put(badgeID, true)
        if (editData(DatabaseContract.TABLE_USER_BADGES_NAME, userID, map)) {
            return FirebaseResult(true, R.string.task_page_badge_congratulation)
        } else {
            return FirebaseResult(false, R.string.task_page_badge_congratulation)
        }
    }

    fun getAllBadges(): LiveData<List<Badge>> {
        val mediatorLiveData = MediatorLiveData<List<Badge>>()
        val query = mDatabase.reference.child(DatabaseContract.TABLE_BADGES)
        val data = FirebaseQueryListLiveData(query, QueryType.ALL_TOPICS)
        transformBadges(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformBadges(mediatorLiveData: MediatorLiveData<List<Badge>>, data: FirebaseQueryListLiveData) {
        mediatorLiveData.addSource(data, { t ->
            if (t != null) {
                val list = arrayListOf<Badge>()
                for (date in t) {
                    val badge = date.getValue(Badge::class.java)
                    if (badge != null) {
                        badge.badgeName = date.key
                        list.add(badge)
                    }
                }
                mediatorLiveData.postValue(list)
            }
        })
    }

    fun getAllUserBadges(userID: String): LiveData<List<String>> {
        val mediatorLiveData = MediatorLiveData<List<String>>()
        val query = mDatabase.reference.child(DatabaseContract.TABLE_USER_BADGES_NAME).child(userID)
        val data = FirebaseQueryListLiveData(query, QueryType.ALL_TOPICS)
        transformUserBadges(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformUserBadges(mediatorLiveData: MediatorLiveData<List<String>>, data: FirebaseQueryListLiveData) {
        mediatorLiveData.addSource(data, { t ->
            if (t != null) {
                val list = arrayListOf<String>()
                for (date in t) {
                    val badgeName = date.key
                    if (badgeName != null) {
                        list.add(badgeName)
                    }
                }
                mediatorLiveData.postValue(list)
            }
        })
    }


    fun getUserNumberOfFinished(exerciseID: String?, taskID: String): LiveData<Int> {
        val mediatorLiveData = MediatorLiveData<Int>()
        val query = mDatabase.reference.child(DatabaseContract.TABLE_USERS_NAME)
                .child(FirebaseAuth.getInstance().currentUser?.uid)
                .child("exercises").child(exerciseID).child(taskID)
        val data = FirebaseQueryLiveData(query)
        transformUserFinished(mediatorLiveData, data)
        return mediatorLiveData
    }

    private fun transformUserFinished(mediatorLiveData: MediatorLiveData<Int>, data: FirebaseQueryLiveData) {
        mediatorLiveData.addSource(data, { t: DataSnapshot? ->
            if (t != null) {
                val hravo = t.getValue(Int::class.java)
                mediatorLiveData.postValue(hravo)
            }
        })
    }

    suspend fun editExercise(taskID: String?, description: String, options: HashMap<String, Boolean>): FirebaseResult {
        if (taskID == null || options.isEmpty()) {
            return FirebaseResult(false, R.string.task_admin_page_task_edit_error)
        }

        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_TASKS_DESCRIPTION, description)
        map.put(DatabaseContract.TABLE_TASKS_CHOICES, options)

        if (editData(DatabaseContract.TABLE_TASKS_NAME, taskID, map)) {
            return FirebaseResult(true, R.string.task_admin_page_task_edit)
        } else {
            return FirebaseResult(false, R.string.task_admin_page_task_edit_error)
        }

    }

    fun getTaskFromId(taskID: String): LiveData<Task> {
        val mediatorLiveData = MediatorLiveData<Task>()
        val query = mDatabase.reference.child(DatabaseContract.TABLE_TASKS_NAME).child(taskID)
        val data = FirebaseQueryLiveData(query)
        transformTask(mediatorLiveData, data)
        return mediatorLiveData
    }

    fun transformTask(mediatorLiveData: MediatorLiveData<Task>, firebaseResult: FirebaseQueryLiveData) {
        mediatorLiveData.addSource(firebaseResult, { t: DataSnapshot? ->
            if (t != null) {
                val task = Task()
                task.description = t.child("description").getValue(String::class.java)
                task.exercise = t.child("exercise").getValue(String::class.java)
                task.photoURL = t.child("photoURL").getValue(String::class.java)
                t.child("choices").children.forEach { snapshot ->
                    val boolValue = snapshot.value
                    if (boolValue is Boolean) {
                        task?.choices?.put(snapshot.key.toString(), boolValue)
                    }
                }
                mediatorLiveData.postValue(task)
            }
        })
    }

    suspend fun setPoints(points: Double, userID: String, exerciseID: String?, taskID: String, numberOfTries: Int): FirebaseResult {
        if (exerciseID == null) {
            return FirebaseResult(false, R.string.task_page_points_set_problem)
        }
        val map = HashMap<String, Any>()
        map.put(DatabaseContract.TABLE_USERS_SCORE, points)

        // Update score of user
        if (editData(DatabaseContract.TABLE_USERS_NAME, userID, map)) {

            val mapper = HashMap<String, Any>()
            val actualTime = Date().time
            mapper.put(taskID, numberOfTries)

            // Add task to finished exercise to user profile
            if (editData(DatabaseContract.TABLE_USERS_NAME + "/" + userID + "/" + DatabaseContract.TABLE_USERS_EXERCISES, exerciseID, mapper)) {
                // Add time to lastTask you took
                val mapData = HashMap<String, Any>()
                mapData.put(DatabaseContract.TABLE_USERS_LAST_TASK, actualTime)
                if (editData(DatabaseContract.TABLE_USERS_NAME, userID, mapData)) {
                    return FirebaseResult(true, R.string.task_page_points_set)
                } else {
                    return FirebaseResult(false, R.string.task_page_points_set_problem)
                }
            } else {
                return FirebaseResult(false, R.string.task_page_points_set_problem)
            }
        } else {
            return FirebaseResult(false, R.string.task_page_points_set_problem)
        }
    }


    suspend fun deleteTask(taskID: String?): FirebaseResult {
        if (taskID == null) {
            return FirebaseResult(false, R.string.task_admin_page_task_deleted_error)
        }

        if (deleteData(DatabaseContract.TABLE_TASKS_NAME, taskID)) {
            return FirebaseResult(true, R.string.task_admin_page_task_deleted)
        } else {
            return FirebaseResult(false, R.string.task_admin_page_task_deleted_error)
        }
    }

}